@@ -2,13 +2,16 @@
 # Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
 # Distributed under the terms of "New BSD License", see the LICENSE file.
 import os
-import h5py
 import shutil
+from argparse import ArgumentParser, Namespace
+
+from h5io_browser.base import _open_hdf
+
 from pyiron_base import Project
 from pyiron_base.state import state
 
 
-def register(parser):
+def register(parser: ArgumentParser) -> None:
     parser.add_argument(
         "-i",
         "--input-path",
@@ -21,8 +24,8 @@ def register(parser):
     )
 
 
-def main(args):
-    with h5py.File(args.input_path, mode="r") as f:
+def main(args: Namespace) -> None:
+    with _open_hdf(filename=args.input_path, mode="r") as f:
         job_name = list(f.keys())[0]
     project_path = os.path.join(os.path.abspath("."), job_name + ".h5")
     shutil.copy(args.input_path, project_path)
