@@ -3,10 +3,10 @@
 # Distributed under the terms of "New BSD License", see the LICENSE file.
 """Interface for classes to serialize to HDF5."""
 
-from pyiron_base.storage.hdfio import ProjectHDFio
-
 from abc import ABC, abstractmethod
 
+from pyiron_base.storage.hdfio import ProjectHDFio
+
 __author__ = "Marvin Poul"
 __copyright__ = (
     "Copyright 2021, Max-Planck-Institut für Eisenforschung GmbH - "
@@ -172,13 +172,26 @@ class HasHDF(ABC):
         """
         return {}
 
-    def _store_type_to_hdf(self, hdf: ProjectHDFio):
-        hdf["NAME"] = self.__class__.__name__
-        hdf["TYPE"] = str(type(self))
-        hdf["OBJECT"] = hdf["NAME"]  # unused alias
+    def _type_to_dict(self):
+        # Needed for the HasDictfromHDF/HasHDFfromDict classes.  When an object
+        # derives from from both them and HasHDF/HasDict it will generally need
+        # HDF_VERSION and DICT_VERSION defined for the version checking inside
+        # from_dict/from_hdf to work properly.  So the code below tries to
+        # escalate to super in case this is the case and falls back to {} if it
+        # is not
+        try:
+            type_dict = super()._type_to_dict()
+        except AttributeError:
+            type_dict = {}
+        type_dict |= {
+            "NAME": self.__class__.__name__,
+            "TYPE": str(type(self)),
+            "OBJECT": self.__class__.__name__,  # unused alias
+            "HDF_VERSION": self.__hdf_version__,
+        }
         if hasattr(self, "__version__"):
-            hdf["VERSION"] = self.__version__
-        hdf["HDF_VERSION"] = self.__hdf_version__
+            type_dict["VERSION"] = self.__version__
+        return type_dict
 
     def from_hdf(self, hdf: ProjectHDFio, group_name: str = None):
         """
@@ -217,7 +230,7 @@ class HasHDF(ABC):
             ):
                 raise ValueError("HDF group must be empty when group_name is not set.")
             self._to_hdf(hdf)
-            self._store_type_to_hdf(hdf)
+            hdf.write_dict_to_hdf(data_dict=self._type_to_dict())
 
     def rewrite_hdf(self, hdf: ProjectHDFio, group_name: str = None):
         """
