@@ -945,7 +945,7 @@ class TableJob(GenericJob):
             self.project.db.item_update({"timestart": datetime.now()}, self.job_id)
         with self.project_hdf5.open("input") as hdf5_input:
             if self._executor_type is None and self.server.cores > 1:
-                self._executor_type = "executorlib.Executor"
+                self._executor_type = "executorlib.SingleNodeExecutor"
             if self._executor_type is not None:
                 with self._get_executor(max_workers=self.server.cores) as exe:
                     self._pyiron_table.create_table(
