@@ -15,6 +15,7 @@ from typing import Optional, Tuple, Union
 
 import monty.io
 import psutil
+from pyiron_snippets.logger import logger
 
 from pyiron_base.database.sqlcolumnlength import JOB_STR_LENGTH
 from pyiron_base.utils.instance import static_isinstance
@@ -42,7 +43,9 @@ _special_symbol_replacements = {
 
 
 def _copy_database_entry(
-    new_job_core: "pyiron_base.jobs.job.generic.GenericJob", job_copied_id: int
+    new_job_core: "pyiron_base.jobs.job.generic.GenericJob",
+    job_copied_id: int,
+    username: Optional[str] = None,
 ) -> None:
     """
     Copy database entry from previous job
@@ -50,6 +53,7 @@ def _copy_database_entry(
     Args:
         new_job_core (GenericJob): Copy of the job object
         job_copied_id (int): Job id of the copied job
+        username (str): Optional name of the user to copy the job to
     """
     db_entry = new_job_core.project.db.get_item_by_id(job_copied_id)
     if db_entry is not None:
@@ -57,6 +61,8 @@ def _copy_database_entry(
         db_entry["subjob"] = new_job_core.project_hdf5.h5_path
         db_entry["project"] = new_job_core.project_hdf5.project_path
         db_entry["projectpath"] = new_job_core.project_hdf5.root_path
+        if username is not None:
+            db_entry["username"] = username
         del db_entry["id"]
         job_id = new_job_core.project.db.add_item_dict(db_entry)
         new_job_core.reset_job_id(job_id=job_id)
@@ -333,6 +339,8 @@ def _job_compress(
     if not _job_is_compressed(job):
         for name in files_to_remove:
             delete_file_or_folder(fullname=os.path.join(job.working_directory, name))
+        if len(files_to_compress) == 0:
+            return
         cwd = os.getcwd()
         try:
             os.chdir(job.working_directory)
@@ -348,7 +356,7 @@ def _job_compress(
         finally:
             os.chdir(cwd)
     else:
-        job.logger.info("The files are already compressed!")
+        logger.info("The files are already compressed!")
 
 
 def _job_decompress(job: "pyiron_base.jobs.job.core.JobCore") -> None:
