@@ -8,15 +8,22 @@ Data structure for versatile data handling.
 import copy
 import json
 import warnings
-from collections.abc import Sequence, Set, Mapping, MutableMapping
+from collections.abc import Mapping, MutableMapping, Sequence, Set
 
 import numpy as np
 import pandas
 
-from pyiron_base.storage.fileio import read, write
-from pyiron_base.storage.hdfstub import HDFStub
+from pyiron_base.interfaces.has_dict import (
+    HasDict,
+    HasDictfromHDF,
+    _from_dict_children,
+    _to_dict_children,
+)
 from pyiron_base.interfaces.has_groups import HasGroups
 from pyiron_base.interfaces.has_hdf import HasHDF
+from pyiron_base.interfaces.lockable import Lockable, sentinel
+from pyiron_base.storage.fileio import read, write
+from pyiron_base.storage.hdfstub import HDFStub, to_object
 
 __author__ = "Marvin Poul"
 __copyright__ = (
@@ -30,7 +37,16 @@ __status__ = "production"
 __date__ = "Jun 17, 2020"
 
 
-_internal_hdf_nodes = ["NAME", "TYPE", "OBJECT", "VERSION", "HDF_VERSION", "READ_ONLY"]
+_internal_hdf_nodes = [
+    "NAME",
+    "TYPE",
+    "OBJECT",
+    "VERSION",
+    "HDF_VERSION",
+    "DICT_VERSION",
+    "READ_ONLY",
+    "KEY_ORDER",
+]
 
 
 def _normalize(key):
@@ -46,16 +62,16 @@ def _normalize(key):
     return key
 
 
-class DataContainer(MutableMapping, HasGroups, HasHDF):
+class DataContainerBase(MutableMapping, Lockable, HasGroups):
     """
     Mutable sequence with optional keys.
 
-    If no argument is given, the constructor creates a new empty DataContainer.  If
+    If no argument is given, the constructor creates a new empty DataContainerBase.  If
     specified init maybe a Sequence, Set or Mapping and all recursive
-    occurrences of these are also wrapped by DataContainer.
+    occurrences of these are also wrapped by DataContainerBase.
 
-    >>> pl = DataContainer([3, 2, 1, 0])
-    >>> pm = DataContainer({"foo": 24, "bar": 42})
+    >>> pl = DataContainerBase([3, 2, 1, 0])
+    >>> pm = DataContainerBase({"foo": 24, "bar": 42})
 
     Access can be like a normal list with integers or optionally with strings
     as keys.
@@ -69,23 +85,23 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
 
     Keys do not have to be present for all elements.
 
-    >>> pl2 = DataContainer([1,2])
+    >>> pl2 = DataContainerBase([1,2])
     >>> pl2["end"] = 3
     >>> pl2
-    DataContainer({0: 1, 1: 2, 'end': 3})
+    DataContainerBase({0: 1, 1: 2, 'end': 3})
 
-    It is also allowed to set an item one past the length of the DataContainer,
+    It is also allowed to set an item one past the length of the DataContainerBase,
     this is then equivalent to appending that element.  This allows to use the
-    update method also with other DataContainers
+    update method also with other DataContainerBases
 
     >>> pl[len(pl)] = -1
     >>> pl
-    DataContainer([3, 2, 1, 0, -1])
+    DataContainerBase([3, 2, 1, 0, -1])
     >>> pl.pop(-1)
     -1
 
     Where strings are used they may also be used as attributes.  Getting keys
-    which clash with methods of DataContainer must be done with item access, but
+    which clash with methods of DataContainerBase must be done with item access, but
     setting them works without overwriting the instance methods, but is not
     recommended for readability.
 
@@ -93,13 +109,13 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
     24
     >>> pm.tail = 23
     >>> pm
-    DataContainer({'foo': 24, 'bar': 42, 'tail': 23})
+    DataContainerBase({'foo': 24, 'bar': 42, 'tail': 23})
 
-    Keys and indices can be tuples to traverse nested DataContainers.
+    Keys and indices can be tuples to traverse nested DataContainerBases.
 
-    >>> pn = DataContainer({"foo": {"bar": [4, 2]}})
+    >>> pn = DataContainerBase({"foo": {"bar": [4, 2]}})
     >>> pn["foo", "bar"]
-    DataContainer([4, 2])
+    DataContainerBase([4, 2])
     >>> pn["foo", "bar", 0]
     4
 
@@ -117,7 +133,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
 
     >>> pl["0"] == pl[0]
     True
-    >>> DataContainer({1: 42})
+    >>> DataContainerBase({1: 42})
     Traceback (most recent call last):
         File "<stdin>", line 1, in <module>
         File "datacontainer.py", line 126, in __init__
@@ -126,11 +142,11 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
 
     When initializing from a dict, it may not have integers or decimal strings
     as keys unless they match their position in the insertion order.  This is
-    to avoid ambiguities in the final order of the DataContainer.
+    to avoid ambiguities in the final order of the DataContainerBase.
 
-    >>> DataContainer({0: "foo", 1: "bar", 2: 42})
-    DataContainer(['foo', 'bar', 42])
-    >>> DataContainer({0: "foo", 2: 42, 1: "bar"})
+    >>> DataContainerBase({0: "foo", 1: "bar", 2: 42})
+    DataContainerBase(['foo', 'bar', 42])
+    >>> DataContainerBase({0: "foo", 2: 42, 1: "bar"})
     Traceback (most recent call last):
         File "<stdin>", line 1, in <module>
         File "datacontainer.py", line 132, in __init__
@@ -138,7 +154,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
     ValueError: keys in initializer must not be int or str of decimal digits or in correct order, is 2
 
 
-    Using keys is completely optional, DataContainer can always be treated as a
+    Using keys is completely optional, DataContainerBase can always be treated as a
     list, with the exception that `iter()` iterates of the keys and indices.
     This is to correctly implement the MutableMapping protocol, to convert to a
     normal list and discard the keys use `values()`.
@@ -156,23 +172,17 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
 
     Implements :class:`.HasGroups`.  Groups are nested data containers and nodes are everything else.
 
-    >>> p = DataContainer({"a": 42, "b": [0, 1, 2]})
+    >>> p = DataContainerBase({"a": 42, "b": [0, 1, 2]})
     >>> p.list_groups()
     ['b']
     >>> p.list_nodes()
     ['a']
 
-    If instantiated with the argument `lazy=True`, data read from HDF5 later via :method:`.from_hdf` are not actually
-    read, but only earmarked to be read later when actually accessed via :class:`.HDFStub`.  This is largely
-    transparent, i.e. when accessing an earmarked value it will automatically be loaded and this loaded value is stored
-    in container.  The only difference is in the string representation of the container, values not read yet appear as
-    'HDFStub(...)' in the output.
-
     .. attention:: Subclasses beware!
 
-        DataContainer require some careful treatment when creating subclasses.
+        DataContainerBase require some careful treatment when creating subclasses.
 
-        1. Since DataContainers are expected to recursively instantiate themselves subclasses need to keep their
+        1. Since DataContainerBases are expected to recursively instantiate themselves subclasses need to keep their
         `__init__ compatible to the base class.  That means being able to be instantiated without arguments, if
         arguments are given the first one (or `init`) has to accept a Mapping or Iterable.  Additional arguments may be
         added, but must be after `init` and must have a default.
@@ -181,18 +191,17 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         `object.__setattr__` the first time you define that attribute.  Afterwards using normal assignment syntax works.
 
         3. Subclasses should always be thought of as general data structures, if you want to subclass to have access to
-        the HDF5 functionality or the way the DataContainer is shown in jupyter notebooks, but only have a fixed number
-        of attributes it is better to create a new class that has an DataContainer as an attribute and dispatch to the
-        :meth:`DataContainer.from_hdf`, :meth:`DataContainer.to_hdf` and :meth:`DataContainer._repr_json_`
+        the HDF5 functionality or the way the DataContainerBase is shown in jupyter notebooks, but only have a fixed number
+        of attributes it is better to create a new class that has an DataContainerBase as an attribute and dispatch to the
+        :meth:`DataContainerBase.from_hdf`, :meth:`DataContainerBase.to_hdf` and :meth:`DataContainerBase._repr_json_`
         methods.
-        4. To allow lazy loading sub classes must accept a `lazy` keyword argument and pass it to `super().__init__`.
 
 
     A few examples for subclasses
 
-    >>> class ExtendedContainer(DataContainer):
-    ...     def __init__(self, init=None, my_fancy_field=42, table_name=None, lazy=False):
-    ...         super().__init__(init=init, table_name=table_name, lazy=lazy)
+    >>> class ExtendedContainer(DataContainerBase):
+    ...     def __init__(self, init=None, my_fancy_field=42, table_name=None):
+    ...         super().__init__(init=init, table_name=table_name)
     ...         object.__setattr__(self, "my_fancy_field", my_fancy_field)
 
     After defining it once like this you can access my_fancy_field as a normal attribute, but it will not be stored in
@@ -209,7 +218,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
     >>> e
     ExtendedContainer({'foo': 1, 'bar': 5})
 
-    Be aware the :class:`.DataContainer` and its subclasses are recursive data structures, i.e. your fancy attribute
+    Be aware the :class:`.DataContainerBase` and its subclasses are recursive data structures, i.e. your fancy attribute
     will be available also on sub groups.
 
     >>> g = e.create_group('sub')
@@ -219,7 +228,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
     >>> e
     ExtendedContainer({'foo': 1, 'bar': 5, 'sub': ExtendedContainer({'fnord': 23})})
 
-    For that reason most of time you'll actually want a class that uses a DataContainer for storage, but doesn't derive
+    For that reason most of time you'll actually want a class that uses a DataContainerBase for storage, but doesn't derive
     from it.
 
     >>> from pyiron_base.interfaces.object import HasStorage
@@ -242,7 +251,6 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
     """
 
     __version__ = "0.1.0"
-    __hdf_version__ = "0.2.0"
 
     def __new__(cls, *args, **kwargs):
         instance = super().__new__(cls)
@@ -252,12 +260,16 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         object.__setattr__(instance, "_store", [])
         object.__setattr__(instance, "_indices", {})
         object.__setattr__(instance, "table_name", None)
-        object.__setattr__(instance, "_read_only", False)
-        object.__setattr__(instance, "_lazy", False)
 
         return instance
 
-    def __init__(self, init=None, table_name=None, lazy=False, wrap_blacklist=()):
+    def __init__(
+        self,
+        init=None,
+        table_name=None,
+        wrap_blacklist=(),
+        lock_method="warning",
+    ):
         """
         Create new container.
 
@@ -265,12 +277,11 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
             init (Sequence, Mapping): initial data for the container, nested occurances of Sequence and Mapping are
                                       translated to nested containers
             table_name (str): default name of the data container in HDF5
-            lazy (bool): if True, use :class:`.HDFStub` to load values lazily from HDF5
             wrap_blacklist (tuple of types): any values in `init` that are instances of the given types are *not*
-                                             wrapped in :class:`.DataContainer`
+                                             wrapped in :class:`.DataContainerBase`
         """
+        super().__init__(lock_method=lock_method)
         self.table_name = table_name
-        self._lazy = lazy
         if init is not None:
             self.update(init, wrap=True, blacklist=wrap_blacklist)
 
@@ -294,33 +305,21 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
 
         elif isinstance(key, int):
             try:
-                v = self._store[key]
-                if not isinstance(v, HDFStub):
-                    return v
-                else:
-                    v = self._store[key] = v.load()
-                    return v
+                return self._store[key]
             except IndexError:
                 raise IndexError("list index out of range") from None
 
         elif isinstance(key, str):
             try:
-                v = self._store[self._indices[key]]
-                if not isinstance(v, HDFStub):
-                    return v
-                else:
-                    v = self._store[self._indices[key]] = v.load()
-                    return v
+                return self._store[self._indices[key]]
             except KeyError:
                 raise KeyError(repr(key)) from None
 
         else:
             raise ValueError("{} is not a valid key, must be str or int".format(key))
 
+    @sentinel
     def __setitem__(self, key, val):
-        if self.read_only:
-            self._read_only_error()
-
         key = _normalize(key)
 
         if isinstance(key, tuple):
@@ -347,10 +346,8 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         else:
             raise ValueError("{} is not a valid key, must be str or int".format(key))
 
+    @sentinel
     def __delitem__(self, key):
-        if self.read_only:
-            self._read_only_error()
-
         key = _normalize(key)
 
         if isinstance(key, tuple):
@@ -387,6 +384,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
     def _is_class_var(cls, name):
         return any(name in c.__dict__ for c in cls.__mro__)
 
+    @sentinel
     def __setattr__(self, name, val):
         # Search instance variables (self.__dict___) and class variables
         # (self.__class__.__dict__ + iterating over mro to find variables on
@@ -398,6 +396,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         else:
             self[name] = val
 
+    @sentinel
     def __delattr__(self, name):
         # see __setattr__
         if name in self.__dict__ or self._is_class_var(name):
@@ -415,7 +414,8 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
     def __repr__(self):
         name = self.__class__.__name__
         if self.has_keys():
-            # access _store and _indices directly to avoid forcing HDFStubs
+            # access _store and _indices directly to avoid forcing HDFStubs in
+            # subclass
             index2key = {v: k for k, v in self._indices.items()}
             return (
                 name
@@ -429,28 +429,6 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         else:
             return name + "([" + ", ".join("{!r}".format(v) for v in self._store) + "])"
 
-    @property
-    def read_only(self):
-        """
-        bool: if set, raise warning when attempts are made to modify the container
-        """
-        return self._read_only
-
-    @read_only.setter
-    def read_only(self, val):
-        # can't mark a read-only list as writeable
-        if self._read_only and not val:
-            self._read_only_error()
-        else:
-            self._read_only = bool(val)
-
-    @classmethod
-    def _read_only_error(cls):
-        warnings.warn(
-            "The input in {} changed, while the state of the job was already "
-            "finished.".format(cls.__name__)
-        )
-
     def to_builtin(self, stringify=False):
         """
         Convert the container back to builtin dict's and list's recursively.
@@ -459,32 +437,20 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
             stringify (bool, optional): convert all non-recursive elements to str
         """
 
-        if self.has_keys():
-            dd = {}
-            for k, v in self.items():
-                # force all string keys in output to work with h5io (it
-                # requires all string keys when storing as json), since
-                # _normalize calls int() on all digit string keys this is
-                # transparent for the rest of the module
-                k = str(k)
-                if isinstance(v, DataContainer):
-                    dd[k] = v.to_builtin(stringify=stringify)
-                else:
-                    dd[k] = repr(v) if stringify else v
+        def rec(v):
+            if isinstance(v, DataContainerBase):
+                return v.to_builtin(stringify=stringify)
+            else:
+                return repr(v) if stringify else v
 
-            return dd
-        elif stringify:
-            return list(
-                v.to_builtin(stringify=stringify)
-                if isinstance(v, DataContainer)
-                else repr(v)
-                for v in self.values()
-            )
+        if self.has_keys():
+            # force all string keys in output to work with h5io (it
+            # requires all string keys when storing as json), since
+            # _normalize calls int() on all digit string keys this is
+            # transparent for the rest of the module
+            return {str(k): rec(v) for k, v in self.items()}
         else:
-            return list(
-                v.to_builtin(stringify=stringify) if isinstance(v, DataContainer) else v
-                for v in self.values()
-            )
+            return [rec(v) for v in self.values()]
 
     # allows "nice" displays in jupyter lab
     def _repr_json_(self):
@@ -566,7 +532,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
             ValueError: if key is found twice and stop_on_first_hit is False
 
         Returns:
-            DataContainer: container that has ``key``
+            DataContainerBase: container that has ``key``
         """
         # search within current level
         if key in self:
@@ -580,11 +546,11 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         # descend into subgroups
         for it in self.groups():
             hit = self[it]._search_parent(key, stop_on_first_hit)
-            if isinstance(hit, DataContainer):
+            if isinstance(hit, DataContainerBase):
                 if stop_on_first_hit:
                     return hit
                 else:
-                    if isinstance(first_hit, DataContainer):
+                    if isinstance(first_hit, DataContainerBase):
                         raise ValueError("'" + key + "' exists more than once!")
                 first_hit = hit
         return first_hit
@@ -596,6 +562,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         else:
             return val
 
+    @sentinel
     def update(self, init, wrap=False, blacklist=(), **kwargs):
         """
         Add all elements or key-value pairs from init to this container.  If wrap is
@@ -604,7 +571,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         Args:
             init (Sequence, Set, Mapping): container to draw new elements from
             wrap (bool): if True wrap all encountered Sequences and Mappings in
-                         :class:`.DataContainer` recursively
+                         :class:`.DataContainerBase` recursively
             blacklist (list of types): when `wrap` is True, don't wrap these types even if they're instances of Sequence
                                        or Mapping
             **kwargs: update from this mapping as well
@@ -639,6 +606,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         else:
             super().update(init, **kwargs)
 
+    @sentinel
     def append(self, val):
         """
         Add new value to the container without a key.
@@ -648,9 +616,10 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         """
         self._store.append(val)
 
+    @sentinel
     def extend(self, vals):
         """
-        Append vals to the end of this DataContainer.
+        Append vals to the end of this DataContainerBase.
 
         Args:
             vals (Sequence): any python sequence to draw new elements from
@@ -659,6 +628,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         for v in vals:
             self.append(v)
 
+    @sentinel
     def insert(self, index, val, key=None):
         """
         Add a new element to the container at the specified position, with an optional
@@ -679,6 +649,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
 
         self._store.insert(index, val)
 
+    @sentinel
     def mark(self, index, key):
         """
         Add a key to an existing item at index.  If key already exists, it is
@@ -691,7 +662,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         Raises:
             IndexError: if index > len(self)
 
-        >>> pl = DataContainer([42])
+        >>> pl = DataContainerBase([42])
         >>> pl.mark(0, "head")
         >>> pl.head == 42
         True
@@ -705,13 +676,15 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
 
         self._indices[key] = index
 
+    @sentinel
     def clear(self):
         """
-        Remove all items from DataContainer.
+        Remove all items from DataContainerBase.
         """
         self._store.clear()
         self._indices.clear()
 
+    @sentinel
     def create_group(self, name):
         """
         Add a new empty subcontainer under the given key.
@@ -720,14 +693,14 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
             name (str): key under which to store the new subcontainer in this container
 
         Returns:
-            DataContainer: the newly created subcontainer
+            DataContainerBase: the newly created subcontainer
 
         Raises:
             ValueError: name already exists in container and is not a sub container
 
-        >>> pl = DataContainer({})
+        >>> pl = DataContainerBase({})
         >>> pl.create_group("group_name")
-        DataContainer([])
+        DataContainerBase([])
         >>> list(pl.group_name)
         []
         """
@@ -739,7 +712,7 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
             if isinstance(v, self.__class__):
                 return v
             else:
-                raise ValueError(f"'{name}' already exists in DataContainer.")
+                raise ValueError(f"'{name}' already exists in DataContainerBase.")
 
     def has_keys(self):
         """
@@ -767,9 +740,9 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         copy module.
 
         Returns:
-            DataContainer: deep copy of itself
+            DataContainerBase: deep copy of itself
 
-        >>> pl = DataContainer([[1,2,3]])
+        >>> pl = DataContainerBase([[1,2,3]])
         >>> pl.copy() == pl
         True
         >>> pl.copy() is pl
@@ -777,9 +750,138 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
         >>> all(a is not b for a, b in zip(pl.copy().values(), pl.values()))
         True
         """
-        self._force_load()
         return copy.deepcopy(self)
 
+    def nodes(self):
+        """
+        Iterator over keys to terminal nodes.
+
+        Returns:
+            :class:`list`: list of keys to normal values.
+        """
+        for k, v in self.items():
+            if not isinstance(v, DataContainerBase):
+                yield k
+
+    def _list_nodes(self):
+        return list(self.nodes())
+
+    def groups(self):
+        """
+        Iterate over keys to nested containers.
+
+        Returns:
+            :class:`list`: list of all keys to elements of :class:`DataContainerBase`.
+        """
+        for k, v in self.items():
+            if isinstance(v, DataContainerBase):
+                yield k
+
+    def _list_groups(self):
+        return list(self.groups())
+
+    @sentinel
+    def read(self, file_name, wrap=True):
+        """
+        Parse file as dictionary and add its keys to this container.
+
+        For supported file types, see :func:`.fileio.read`.
+
+        Errors during reading of the files generate a warning, but leave the container unchanged.
+
+        Args:
+            file_name(str): path to the input file
+            wrap(bool), if set to true will wrap the inputed data itself as a datacontainer inside the datacontainer
+
+        Raises:
+            :class:`ValueError`: if file extension doesn't match one of the supported ones
+        """
+        self.update(read(file_name), wrap=wrap)
+
+    def write(self, file_name):
+        """
+        Writes the DataContainerBase to a text file.
+
+        For supported file types, see :func:`.fileio.write`.
+
+        Args:
+            file_name(str): the name of the file to be writen to.
+        """
+        write(self.to_builtin(), file_name)
+
+    # Lockable overload
+    def _on_unlock(self):
+        from sys import version_info as python_version
+
+        # a little dance to ensure that warning appear at the correct call
+        # site, i.e. where someone either calls unlocked() or sets read_only
+        if python_version[0] == 3 and python_version[1] >= 12:
+            from pyiron_base.interfaces.lockable import __file__ as lock_file
+
+            warnings.warn(
+                "Unlock previously locked object!",
+                skip_file_prefixes=(__file__, lock_file),
+            )
+        else:
+            # stacklevel is so high, because _on_unlock is called after several
+            # layers of Lockable and DataContainer.__setattr__ when used to set
+            # read_only; when used with unlocked() a fixed stack level still
+            # emits it at the wrong place, but we cannot do better without
+            # Python 3.12
+            warnings.warn("Unlock previously locked object!", stacklevel=5)
+        super()._on_unlock()
+
+
+class DataContainer(DataContainerBase, HasHDF, HasDict):
+    __dict_version__ = "0.2.0"
+    __doc__ = f"""{DataContainerBase.__doc__}
+
+    If instantiated with the argument `lazy=True`, data read from HDF5 later via :method:`.from_hdf` are not actually
+    read, but only earmarked to be read later when actually accessed via :class:`.HDFStub`.  This is largely
+    transparent, i.e. when accessing an earmarked value it will automatically be loaded and this loaded value is stored
+    in container.  The only difference is in the string representation of the container, values not read yet appear as
+    'HDFStub(...)' in the output.
+
+    .. attention:: Subclasses beware!
+        1. To allow lazy loading sub classes must accept a `lazy` keyword argument and pass it to `super().__init__`.
+    """
+    __hdf_version__ = "0.2.0"
+
+    def __new__(cls, *args, **kwargs):
+        instance = super().__new__(cls, *args, **kwargs)
+        object.__setattr__(instance, "_lazy", False)
+
+        return instance
+
+    def __init__(
+        self,
+        init=None,
+        table_name=None,
+        lazy=False,
+        wrap_blacklist=(),
+        lock_method="warning",
+    ):
+        """
+        Create new container.
+
+        Args:
+            init (Sequence, Mapping): initial data for the container, nested occurances of Sequence and Mapping are
+                                      translated to nested containers
+            table_name (str): default name of the data container in HDF5
+            lazy (bool): if True, use :class:`.HDFStub` to load values lazily from HDF5
+            wrap_blacklist (tuple of types): any values in `init` that are instances of the given types are *not*
+                                             wrapped in :class:`.DataContainerBase`
+        """
+        super().__init__(
+            init=init,
+            table_name=table_name,
+            wrap_blacklist=wrap_blacklist,
+            lock_method=lock_method,
+        )
+        self._lazy = lazy
+
+    # HasHDF impl'
+
     def _get_hdf_group_name(self):
         return self.table_name
 
@@ -812,110 +914,64 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
                         "Error saving {} (key {}): DataContainer doesn't support saving elements "
                         'of type "{}" to HDF!'.format(v, k, type(v))
                     ) from None
-        for n in hdf.list_nodes():
+        for n in hdf.list_nodes() + hdf.list_groups():
             if n not in written_keys:
                 del hdf[n]
-        for g in hdf.list_groups():
-            if g not in written_keys:
-                del hdf[g]
 
     def _from_hdf(self, hdf, version=None):
-        self.clear()
+        with self.unlocked():
+            self.clear()
 
-        if version == "0.1.0":
-            self.update(hdf["data"], wrap=True)
-            self.read_only = bool(hdf.get("read_only", False))
-        else:
+            if version == "0.1.0":
+                self.update(hdf["data"], wrap=True)
+                self.read_only = bool(hdf.get("read_only", False))
+            else:
 
-            def normalize_key(name):
-                # split a dataset/group name into the position in the list and
-                # the key
-                if "__index_" in name:
-                    k, i = name.split("__index_", maxsplit=1)
-                else:
-                    k = name
-                    i = -1
-                i = int(i)
-                if k == "":
-                    return i, i
-                else:
-                    return i, k
+                def normalize_key(name):
+                    # split a dataset/group name into the position in the list and
+                    # the key
+                    if "__index_" in name:
+                        k, i = name.split("__index_", maxsplit=1)
+                    else:
+                        k = name
+                        i = -1
+                    i = int(i)
+                    if k == "":
+                        return i, i
+                    else:
+                        return i, k
 
-            items = []
-            for n in hdf.list_nodes():
-                if n in _internal_hdf_nodes:
-                    continue
-                items.append(
-                    (*normalize_key(n), hdf[n] if not self._lazy else HDFStub(hdf, n))
-                )
-            for g in hdf.list_groups():
-                items.append(
-                    (
-                        *normalize_key(g),
-                        hdf[g].to_object() if not self._lazy else HDFStub(hdf, g),
+                items = []
+                for n in hdf.list_nodes():
+                    if n in _internal_hdf_nodes:
+                        continue
+                    items.append(
+                        (
+                            *normalize_key(n),
+                            hdf[n] if not self._lazy else HDFStub(hdf, n),
+                        )
+                    )
+                for g in hdf.list_groups():
+                    items.append(
+                        (
+                            *normalize_key(g),
+                            to_object(hdf[g]) if not self._lazy else HDFStub(hdf, g),
+                        )
                     )
-                )
 
-            for _, k, v in sorted(items, key=lambda x: x[0]):
-                self[k] = v
+                for _, k, v in sorted(items, key=lambda x: x[0]):
+                    self[k] = v
 
-            self.read_only = bool(hdf.get("READ_ONLY", False))
+                self.read_only = bool(hdf.get("READ_ONLY", False))
 
-    def nodes(self):
-        """
-        Iterator over keys to terminal nodes.
-
-        Returns:
-            :class:`list`: list of keys to normal values.
-        """
-        for k, v in self.items():
-            if not isinstance(v, DataContainer):
-                yield k
-
-    def _list_nodes(self):
-        return list(self.nodes())
-
-    def groups(self):
-        """
-        Iterate over keys to nested containers.
-
-        Returns:
-            :class:`list`: list of all keys to elements of :class:`DataContainer`.
-        """
-        for k, v in self.items():
-            if isinstance(v, DataContainer):
-                yield k
-
-    def _list_groups(self):
-        return list(self.groups())
-
-    def read(self, file_name, wrap=True):
-        """
-        Parse file as dictionary and add its keys to this container.
-
-        For supported file types, see :func:`.fileio.read`.
-
-        Errors during reading of the files generate a warning, but leave the container unchanged.
-
-        Args:
-            file_name(str): path to the input file
-            wrap(bool), if set to true will wrap the inputed data itself as a datacontainer inside the datacontainer
-
-        Raises:
-            :class:`ValueError`: if file extension doesn't match one of the supported ones
-        """
-        self.update(read(file_name), wrap=wrap)
-
-    def write(self, file_name):
-        """
-        Writes the DataContainer to a text file.
-
-        For supported file types, see :func:`.fileio.write`.
-
-        Args:
-            file_name(str): the name of the file to be writen to.
-        """
-        write(self.to_builtin(), file_name)
+    # HDFStub compat
+    def __getitem__(self, key):
+        value = super().__getitem__(key)
+        if not isinstance(value, HDFStub):
+            return value
+        else:
+            value = self[key] = value.load()
+            return value
 
     def _force_load(self, recursive=True):
         """
@@ -932,10 +988,75 @@ class DataContainer(MutableMapping, HasGroups, HasHDF):
             if recursive and isinstance(v, DataContainer):
                 v._force_load()
 
+    def copy(self):
+        """
+        Returns deep copy of it self.  A shallow copy can be obtained via the
+        copy module.
+
+        Returns:
+            DataContainer: deep copy of itself
+
+        >>> pl = DataContainer([[1,2,3]])
+        >>> pl.copy() == pl
+        True
+        >>> pl.copy() is pl
+        False
+        >>> all(a is not b for a, b in zip(pl.copy().values(), pl.values()))
+        True
+        """
+        self._force_load()
+        return super().copy()
+
     def __init_subclass__(cls):
         # called whenever a subclass of DataContainer is defined, then register all subclasses with the same function
         # that the DataContainer is registered
         HDFStub.register(cls, lambda h, g: h[g].to_object(lazy=True))
 
+    def to_builtin(self, stringify=False):
+        data = super().to_builtin(stringify=stringify)
+
+        def to(v):
+            if isinstance(v, HasDict):
+                return v.to_dict()
+            elif isinstance(v, HasHDF):
+                return HasDictfromHDF.to_dict(v)
+            else:
+                return v
+
+        if not stringify:
+            if isinstance(data, dict):
+                data = {k: to(v) for k, v in data.items()}
+            elif isinstance(data, list):
+                data = [to(v) for v in data]
+            else:
+                assert False, "to_builtin returned neither list nor dict"
+
+        return data
+
+    def _to_dict(self):
+        # stringify keys in case we are acting like a list
+        data = {str(k): v for k, v in dict(self).items()}
+        order = list(data)
+        data["READ_ONLY"] = self.read_only
+        data["KEY_ORDER"] = order
+        return _to_dict_children(data)
+
+    def _from_dict(self, obj_dict, version=None):
+        obj_dict = _from_dict_children(obj_dict)
+        if version == "0.2.0":
+            order = obj_dict.pop("KEY_ORDER")
+        else:
+            order = None
+        self.read_only = obj_dict.pop("READ_ONLY", False)
+        for key in _internal_hdf_nodes:
+            obj_dict.pop(key, None)
+        with self.unlocked():
+            self.clear()
+            if order is not None:
+                for key in order:
+                    self[key] = obj_dict[key]
+            else:
+                self.update(obj_dict)
+
 
 HDFStub.register(DataContainer, lambda h, g: h[g].to_object(lazy=True))
